@@ -631,30 +631,32 @@ int my_slider_proc(int msg, DIALOG *d, int c)
  */
 int my_edit_proc(int msg, DIALOG *d, int c)
 {
-   int f, l, p, w, x, fg;
-   int b; 
-   int scroll; 
-   char *s;
-   char buf[2];
+   static int ignore_next_uchar = FALSE;
+   int last_was_space, new_pos, i, k;
+   int f, l, p, w, x, fg, b, scroll;
+   char buf[16];
+   char *s, *t;
+   int rtm;
 
    s = d->dp;
-   l = strlen(s);
+   l = ustrlen(s);
    if (d->d2 > l) 
       d->d2 = l;
 
-  /* calculate maximal number of displayable characters */
-   b = x = 0;
+   /* calculate maximal number of displayable characters */
    if (d->d2 == l)  {
-      buf[0] = ' ';
-      buf[1] = 0;
+      usetc(buf+usetc(buf, ' '), 0);
       x = text_length(font, buf);
    }
+   else
+      x = 0;
+
+   b = 0;
 
-   buf[1] = 0;
    for (p=d->d2; p>=0; p--) {
-      buf[0] = s[p];
-      b++;
+      usetc(buf+usetc(buf, ugetat(s, p)), 0);
       x += text_length(font, buf);
+      b++;
       if (x > d->w) 
 	 break;
    }
@@ -686,18 +688,26 @@ int my_edit_proc(int msg, DIALOG *d, int c)
 	    p = 0; 
 
 	 for (; p<=b; p++) {
-	    buf[0] = s[p] ? s[p] : ' ';
+	    f = ugetat(s, p);
+	    usetc(buf+usetc(buf, (f) ? f : ' '), 0);
 	    w = text_length(font, buf);
 	    if (x+w > d->w) 
 	       break;
 	    f = ((p == d->d2) && (d->flags & D_GOTFOCUS));
-       text_mode(f ? d->fg : d->bg); /* modif de chez tonton */
-       textout(screen, font, buf, d->x+x, d->y, f ? d->bg : d->fg);
-       text_mode(-1);
-	    textout(screen, font, buf, d->x+x, d->y, -1);  /* idem chez tata*/
+	    rtm = text_mode((f) ? fg : d->bg);
+	    textout(screen, font, buf, d->x+x, d->y, (f) ? d->bg : fg);
+	    /*
+	     * Now we redraw with color -1 to use the font's native look
+	     */
+	    text_mode(-1);
+	    textout(screen, font, buf, d->x+x, d->y, -1);
+	    /*
+	     * End of the ugly patch
+	     */
+	    text_mode(rtm);
 	    x += w;
 	 }
-	 if (x<d->w)
+	 if (x < d->w)
 	    rectfill(screen, d->x+x, d->y, d->x+d->w-1, d->y+text_height(font)-1, d->bg);
 	 break;
 
@@ -712,15 +722,15 @@ int my_edit_proc(int msg, DIALOG *d, int c)
 	    p = 0; 
 
 	 for (; p<b; p++) {
-	    buf[0] = s[p];
+	    usetc(buf+usetc(buf, ugetat(s, p)), 0);
 	    x += text_length(font, buf);
-	    if (x > mouse_x) 
+	    if (x > gui_mouse_x()) 
 	       break;
 	 }
 	 d->d2 = MID(0, p, l);
-	 show_mouse(NULL);
-	 SEND_MESSAGE(d, MSG_DRAW, 0);
-	 show_mouse(screen);
+	 scare_mouse();
+	 object_message(d, MSG_DRAW, 0);
+	 unscare_mouse();
 	 break;
 
       case MSG_WANTFOCUS:
@@ -729,11 +739,41 @@ int my_edit_proc(int msg, DIALOG *d, int c)
 	 return D_WANTFOCUS;
 
       case MSG_CHAR:
+	 ignore_next_uchar = FALSE;
+
 	 if ((c >> 8) == KEY_LEFT) {
-	    if (d->d2 > 0) d->d2--;
+	    if (d->d2 > 0) {
+	       if (key_shifts & KB_CTRL_FLAG) {
+		  last_was_space = TRUE;
+		  new_pos = 0;
+		  t = s;
+		  for (i = 0; i < d->d2; i++) {
+		     k = ugetx(&t);
+		     if (uisspace(k))
+			last_was_space = TRUE;
+		     else if (last_was_space) {
+			last_was_space = FALSE;
+			new_pos = i;
+		     }
+		  }
+		  d->d2 = new_pos;
+	       }
+	       else
+		  d->d2--;
+	    }
 	 }
 	 else if ((c >> 8) == KEY_RIGHT) {
-	    if (d->d2 < l) d->d2++;
+	    if (d->d2 < l) {
+	       if (key_shifts & KB_CTRL_FLAG) {
+		  t = s + uoffset(s, d->d2);
+		  for (k = ugetx(&t); uisspace(k); k = ugetx(&t))
+		     d->d2++;
+		  for (; k && !uisspace(k); k = ugetx(&t))
+		     d->d2++;
+	       }
+	       else
+		  d->d2++;
+	    }
 	 }
 	 else if ((c >> 8) == KEY_HOME) {
 	    d->d2 = 0;
@@ -743,47 +783,50 @@ int my_edit_proc(int msg, DIALOG *d, int c)
 	 }
 	 else if ((c >> 8) == KEY_DEL) {
 	    if (d->d2 < l)
-	       for (p=d->d2; s[p]; p++) 
-		  s[p] = s[p+1];
+	       uremove(s, d->d2);
 	 }
 	 else if ((c >> 8) == KEY_BACKSPACE) {
 	    if (d->d2 > 0) {
 	       d->d2--;
-	       for (p=d->d2; s[p]; p++) 
-		  s[p] = s[p+1];
+	       uremove(s, d->d2);
 	    }
 	 }
 	 else if ((c >> 8) == KEY_ENTER) {
 	    if (d->flags & D_EXIT) {
-	       show_mouse(NULL);
-	       SEND_MESSAGE(d, MSG_DRAW, 0);
-	       show_mouse(screen);
+	       scare_mouse();
+	       object_message(d, MSG_DRAW, 0);
+	       unscare_mouse();
 	       return D_CLOSE;
 	    }
 	    else
 	       return D_O_K;
 	 }
-	 else {
-	    c &= 0xff;
-	    if ((c >= 32) && (c <= 255)) {
-	       if (l < d->d1) {
-		  while (l >= d->d2) {
-		     s[l+1] = s[l];
-		     l--;
-		  }
-		  s[d->d2] = c;
-		  d->d2++;
-	       }
-	    }
-	    else
-	       return D_O_K;
+	 else if ((c >> 8) == KEY_TAB) {
+	    ignore_next_uchar = TRUE;
+	    return D_O_K;
 	 }
-
-	 /* if we changed something, better redraw... */
-	 show_mouse(NULL);
-	 SEND_MESSAGE(d, MSG_DRAW, 0);
-	 show_mouse(screen);
+	 else {
+	    /* don't process regular keys here: MSG_UCHAR will do that */
+	    break;
+	 }
+	 scare_mouse();
+	 object_message(d, MSG_DRAW, 0);
+	 unscare_mouse();
 	 return D_USED_CHAR;
+
+      case MSG_UCHAR:
+	 if ((c >= ' ') && (uisok(c)) && (!ignore_next_uchar)) {
+	    if (l < d->d1) {
+	       uinsert(s, d->d2, c);
+	       d->d2++;
+
+	       scare_mouse();
+	       object_message(d, MSG_DRAW, 0);
+	       unscare_mouse();
+	    }
+	    return D_USED_CHAR;
+	 }
+	 break;
    }
 
    return D_O_K;
