@@ -65,12 +65,13 @@
 /* fonctions globales                                               */
 /*==================================================================*/
 
-int my_textbox_proc(int , DIALOG *, int );
-int my_button_proc(int , DIALOG *, int );
+int my_textbox_proc(int , DIALOG *, int c);
+int my_button_proc(int , DIALOG *, int c);
 int my_text_proc(int msg, DIALOG *d, int c);
 int my_ctext_proc(int msg, DIALOG *d, int c);
 int my_slider_proc(int msg, DIALOG *d, int c);
 int my_edit_proc(int msg, DIALOG *d, int c);
+int my_list_proc(int msg, DIALOG *d, int c);
 BITMAP *my_create_bitmap (int w, int h);
 
 #endif
