@@ -122,9 +122,6 @@ static void place_team (int part, int team)
 
   fighters=CURRENT_ARMY_SIZE/PLAYING_TEAMS;
   ACTIVE_FIGHTERS[team]=0;
-  /*
-    health=fsqrt(fsqrt(1<<LW_CONFIG_CURRENT_rULES.FIGHTER_NEW_HEALTH))/4;
-    if (health>=MAX_FIGHTER_HEALTH)*/
   health=MAX_FIGHTER_HEALTH-1;
 
   switch (part)
@@ -256,8 +253,12 @@ int create_army (void)
 {
   int ret=-1,i;
 
+  /*
+   * The "battle room" is filled from 2% (1/50 = minimum) to
+   * 66% (33/50 = maximum) depending on the user settings.
+   */
   CURRENT_ARMY_SIZE= (get_battle_room()
-		      *(LW_CONFIG_CURRENT_RULES.fighter_number+1))/64;
+		      *(LW_CONFIG_CURRENT_RULES.fighter_number+1))/50;
 
   CURRENT_ARMY_SIZE/=PLAYING_TEAMS;
   if (CURRENT_ARMY_SIZE<1)
