@@ -54,6 +54,7 @@
 
 #include <stdlib.h>
 #include <allegro.h>
+#include <string.h>
 
 #include "alleg2.h"
 #include "bigdata.h"
@@ -86,117 +87,131 @@ static int BIG_DATA_TABLE[BIG_DATA_NUMBER];
 /*------------------------------------------------------------------*/
 void reset_big_data (void)
 {
- BIG_DATA_BOTTOM=0;
- BIG_DATA_TOP=BIG_DATA_NUMBER-1;
- BIG_DATA_TABLE[BIG_DATA_BOTTOM]=0;
- BIG_DATA_TABLE[BIG_DATA_TOP]=BIG_DATA_SIZE;
+  BIG_DATA_BOTTOM=0;
+  BIG_DATA_TOP=BIG_DATA_NUMBER-1;
+  BIG_DATA_TABLE[BIG_DATA_BOTTOM]=0;
+  BIG_DATA_TABLE[BIG_DATA_TOP]=BIG_DATA_SIZE;
 }
 
 /*------------------------------------------------------------------*/
 void *malloc_big_data (void)
 {
- int nb_try=0;
+  int nb_try=0;
 
- log_print_str ("Allocating ");
- log_print_int(STARTUP_BIG_DATA);
- log_print_str(" kb");
- log_flush();
+  log_print_str ("Allocating ");
+  log_print_int(STARTUP_BIG_DATA);
+  log_print_str(" kb");
+  log_flush();
 
- BIG_DATA_PTR=NULL;
- STARTUP_BIG_DATA+=1024;
+  BIG_DATA_PTR=NULL;
+  STARTUP_BIG_DATA+=1024;
 
- while (STARTUP_BIG_DATA>=1024 &&
-       BIG_DATA_PTR==NULL &&
-       (STARTUP_AUTO || nb_try==0))
-       {
-        nb_try++;
-        STARTUP_BIG_DATA-=1024;
-        BIG_DATA_SIZE=STARTUP_BIG_DATA*1024;
-        BIG_DATA_PTR=malloc(BIG_DATA_SIZE);
-       }
+  while (STARTUP_BIG_DATA>=1024 &&
+	 BIG_DATA_PTR==NULL &&
+	 (STARTUP_AUTO || nb_try==0))
+    {
+      nb_try++;
+      STARTUP_BIG_DATA-=1024;
+      BIG_DATA_SIZE=STARTUP_BIG_DATA*1024;
+      BIG_DATA_PTR=malloc(BIG_DATA_SIZE);
+    }
 
- if (nb_try>1){
+  if (nb_try>1){
     log_print_str ("Last try withg ");
     log_print_int(STARTUP_BIG_DATA);
     log_print_str(" kb");
- }
+  }
 
- if (BIG_DATA_PTR==NULL)
+  if (BIG_DATA_PTR==NULL)
     BIG_DATA_SIZE=0;
 
- display_success (BIG_DATA_SIZE);
- reset_big_data();
+  display_success (BIG_DATA_SIZE);
+  reset_big_data();
 
- return (BIG_DATA_PTR);
+  return (BIG_DATA_PTR);
 }
 
 /*------------------------------------------------------------------*/
 void *malloc_in_big_data_bottom (int size)
 {
- char *ptr;
- int bottom,top,new_offset;
+  char *ptr;
+  int bottom,top,new_offset;
 
- bottom=BIG_DATA_TABLE[BIG_DATA_BOTTOM];
- top=BIG_DATA_TABLE[BIG_DATA_TOP];
- new_offset=((bottom+size+15)/16)*16;
- if (new_offset<top
-    && BIG_DATA_BOTTOM<(BIG_DATA_TOP-1))
+  bottom=BIG_DATA_TABLE[BIG_DATA_BOTTOM];
+  top=BIG_DATA_TABLE[BIG_DATA_TOP];
+  new_offset=((bottom+size+15)/16)*16;
+  if (new_offset<top
+      && BIG_DATA_BOTTOM<(BIG_DATA_TOP-1))
     {
-     ptr=BIG_DATA_PTR;
-     ptr+=bottom;
-     BIG_DATA_TABLE[++BIG_DATA_BOTTOM]=new_offset;
-    }
- else
-   ptr=NULL;
+      ptr=BIG_DATA_PTR;
+      ptr+=bottom;
+      BIG_DATA_TABLE[++BIG_DATA_BOTTOM]=new_offset;
 
- return ptr;
+      /*
+       * This is theorically not needed, but in case there's a
+       * bug in the assembly code for instance, this will increase
+       * the chances that things behave the same on every computer
+       */
+      memset(ptr,0,size);
+    }
+  else
+    ptr=NULL;
+
+  return ptr;
 }
 
 /*------------------------------------------------------------------*/
 void *malloc_in_big_data_top (int size)
 {
- char *ptr;
- int bottom,top,new_offset;
+  char *ptr;
+  int bottom,top,new_offset;
 
- bottom=BIG_DATA_TABLE[BIG_DATA_BOTTOM];
- top=BIG_DATA_TABLE[BIG_DATA_TOP];
- new_offset=((top-size)/16)*16;
- if (new_offset>=bottom
-    && BIG_DATA_BOTTOM<(BIG_DATA_TOP-1))
+  bottom=BIG_DATA_TABLE[BIG_DATA_BOTTOM];
+  top=BIG_DATA_TABLE[BIG_DATA_TOP];
+  new_offset=((top-size)/16)*16;
+  if (new_offset>=bottom
+      && BIG_DATA_BOTTOM<(BIG_DATA_TOP-1))
     {
-     ptr=BIG_DATA_PTR;
-     ptr+=new_offset;
-     BIG_DATA_TABLE[--BIG_DATA_TOP]=new_offset;
+      ptr=BIG_DATA_PTR;
+      ptr+=new_offset;
+      BIG_DATA_TABLE[--BIG_DATA_TOP]=new_offset;
+
+      /*
+       * This is theorically not needed, but in case there's a
+       * bug in the assembly code for instance, this will increase
+       * the chances that things behave the same on every computer
+       */
+      memset(ptr,0,size);
     }
- else
+  else
     ptr=NULL;
 
- return ptr;
+  return ptr;
 }
 
 /*------------------------------------------------------------------*/
 void free_last_big_data_bottom (void)
 {
- if (BIG_DATA_BOTTOM>0)
+  if (BIG_DATA_BOTTOM>0)
     BIG_DATA_BOTTOM--;
 }
 
 /*------------------------------------------------------------------*/
 void free_last_big_data_top (void)
 {
- if (BIG_DATA_TOP<BIG_DATA_NUMBER-1)
+  if (BIG_DATA_TOP<BIG_DATA_NUMBER-1)
     BIG_DATA_TOP++;
 }
 
 /*------------------------------------------------------------------*/
 int check_free_memory (void)
 {
- BITMAP *result;
+  BITMAP *result;
 
- result=my_create_bitmap (MEMORY_TO_LEAVE/2,1024);
- if (result)
+  result=my_create_bitmap (MEMORY_TO_LEAVE/2,1024);
+  if (result)
     destroy_bitmap (result);
 
- return result!=NULL;
+  return result!=NULL;
 }
 
