@@ -115,6 +115,7 @@
 #define LABEL_SERVER_ADDRESS "SERVER_ADDRESS"
 #define LABEL_SERVER_PORT "SERVER_PORT"
 #define LABEL_ASM_ALGORITHM "ASM_ALGORITHM"
+#define LABEL_PASSWORD "PASSWORD"
 
 int CONFIG_GFX_GAME;
 int CONFIG_GFX_MENU;
@@ -158,6 +159,7 @@ int CONFIG_ROUNDS_PER_SEC_LIMIT;
 char CONFIG_SERVER_ADDRESS[SERVER_ADDRESS_SIZE+1];
 int CONFIG_SERVER_PORT;
 int CONFIG_ASM_ALGORITHM;
+char CONFIG_PASSWORD[PASSWORD_SIZE+1];
 
 LW_NETCONF LW_CONFIG_CURRENT_RULES;
 
@@ -216,7 +218,7 @@ void load_default_options (void)
   CONFIG_LEVEL_MAP=0;
   CONFIG_LEVEL_FG=135;
   CONFIG_LEVEL_BG=131;
-  CONFIG_FIGHTER_NUMBER=16;
+  CONFIG_FIGHTER_NUMBER=12;
  
   CONFIG_KEY_UP[0]= KEY_UP;
   CONFIG_KEY_DOWN[0]= KEY_DOWN;
@@ -282,6 +284,8 @@ void load_default_options (void)
   CONFIG_SERVER_PORT=LW_SERVER_DEFAULT_PORT;
 
   CONFIG_ASM_ALGORITHM=1;
+
+  strcpy(CONFIG_PASSWORD,"toto");
 }
 
 /*------------------------------------------------------------------*/
@@ -356,6 +360,9 @@ static int load_disk_options (void)
       CONFIG_SERVER_PORT=get_config_int(LABEL_MAIN,LABEL_SERVER_PORT,CONFIG_SERVER_PORT);
 
       CONFIG_ASM_ALGORITHM=get_config_int(LABEL_MAIN,LABEL_ASM_ALGORITHM,CONFIG_ASM_ALGORITHM);
+
+      strncpy(CONFIG_PASSWORD,get_config_string(LABEL_MAIN,LABEL_PASSWORD,CONFIG_PASSWORD),sizeof(CONFIG_PASSWORD));
+      CONFIG_PASSWORD[sizeof(CONFIG_PASSWORD)-1]='\0';
       return 1;
     }
   else
@@ -449,6 +456,8 @@ int save_config_options (void)
 	
   set_config_int(LABEL_MAIN,LABEL_ASM_ALGORITHM,CONFIG_ASM_ALGORITHM);
 
+  set_config_string(LABEL_MAIN,LABEL_PASSWORD,CONFIG_PASSWORD);
+
   return 0;
 }
 
