@@ -76,13 +76,18 @@
 #include "thrdgen.h"
 #include "chat.h"
 #include "netmess.h"
+#include "sound.h"
 
 /*==================================================================*/
-/* globals                                                          */
+/* constants                                                        */
 /*==================================================================*/
 
 #define CONNECT_BUF_SIZE 200
 
+/*==================================================================*/
+/* structs                                                          */
+/*==================================================================*/
+
 typedef struct
 {
   int sock;
@@ -108,7 +113,8 @@ static void connect_on_server_keepalive(void *args);
 static int update_connect_menu(DIALOG *d,
 			       int waited_teams,
 			       LW_TEAMSTARTINFO *team_start_info,
-			       LW_CHAT_HISTORY *chat_history);
+			       LW_CHAT_HISTORY *chat_history,
+			       int *connected_teams_prev);
 static void position_players_list (DIALOG *d);
 static void position_chat_history (DIALOG *d);
 
@@ -127,6 +133,7 @@ int lw_connect_menu (int sock)
   LW_CONNECT_DATA *data;
   int network_error=0;
   int hide_start_button=0;
+  int connected_teams_prev=-1;
 
   memset(d,0,sizeof(d));
   buf_waited_teams[0]='\0';
@@ -154,7 +161,7 @@ int lw_connect_menu (int sock)
       d[4].proc=my_textbox_proc;
       d[5].proc=my_textbox_proc;
       d[7].proc=my_textbox_proc;
-      d[8].proc=my_textbox_proc;
+      d[8].proc=d_box_proc;
       d[9].proc=my_edit_proc;
 
       position_players_list(d+5);
@@ -171,9 +178,9 @@ int lw_connect_menu (int sock)
       d[5].dp=buf_connected_players;
       d[6].dp=lw_lang_string(LW_LANG_STRING_CONNECT_STARTNOW);
       d[7].dp=buf_chat_history;
-      d[8].dp="";
       d[9].dp=buf_chat_message;
       d[9].d1=LW_CHAT_MESSAGE_SIZE;
+      d[9].flags=D_EXIT;
       d[10].dp=lw_lang_string(LW_LANG_STRING_CONNECT_SENDMESSAGE);
 
       if (hide_start_button)
@@ -212,7 +219,8 @@ int lw_connect_menu (int sock)
 		      update_connect_menu(d,
 					  data->waited_teams,
 					  data->team_start_info,
-					  &(data->chat_history));
+					  &(data->chat_history),
+					  &connected_teams_prev);
 		    }
 		}
 	      data->thread_running=1;
@@ -279,6 +287,7 @@ int lw_connect_menu (int sock)
 		      display_back_image();
 		    }		 
 		  break;
+		case 9:
 		case 10:
 		  if (strlen(d[9].dp) && !data->chat_send)
 		    {
@@ -318,7 +327,8 @@ int lw_connect_menu (int sock)
 static int update_connect_menu(DIALOG *d,
 			       int waited_teams,
 			       LW_TEAMSTARTINFO *team_start_info,
-			       LW_CHAT_HISTORY *chat_history)
+			       LW_CHAT_HISTORY *chat_history,
+			       int *connected_teams_prev)
 {
   int result=0;
   int i;
@@ -328,6 +338,7 @@ static int update_connect_menu(DIALOG *d,
   static int old_chat_length=0;
   int chat_length;
   int l;
+  int connected_teams=0;
 
   sprintf(d[4].dp,"%s%d%s",
 	  lw_lang_string(LW_LANG_STRING_CONNECT_WAITINGFOR),
@@ -352,6 +363,8 @@ static int update_connect_menu(DIALOG *d,
 		  i+1,sep,team_start_info[i].name);
 	  
 	  strcat(d[5].dp,buf);
+
+	  connected_teams++;
 	}
     }
 
@@ -386,6 +399,35 @@ static int update_connect_menu(DIALOG *d,
 
   show_mouse(screen);
 
+  /*
+   * If there are more teams connected than last times, we're likely
+   * to announce it with a sound
+   */
+  if (connected_teams>0 && connected_teams>*connected_teams_prev)
+    {
+      /*
+       * We check if this is not the first time 
+       * (*connected_teams_prev==-1) and in this case we play 
+       * nothing
+       */
+      if ((*connected_teams_prev)>=0)
+	{
+	  play_connect();
+	}
+      (*connected_teams_prev)=connected_teams;
+    }
+  else
+    {
+      /*
+       * We updated the numner of connected teams, but take care
+       * note to overwrite it if it's -1
+       */
+      if ((*connected_teams_prev)>=0)
+	{
+	  (*connected_teams_prev)=connected_teams;
+	}
+    }
+
   return result;
 }
 
