@@ -183,5 +183,12 @@ int error_server_full(void)
 			 (LW_LANG_STRING_ERROR_SERVERFULL)));
 }
 
+/*------------------------------------------------------------------*/
+int error_bad_password(void)
+{
+  return (standard_error(lw_lang_string
+			 (LW_LANG_STRING_ERROR_BADPASSWORD)));
+}
+
 
 
