@@ -66,6 +66,8 @@
 #include "ticker.h"
 #include "sockgen.h"
 #include "lang.h"
+#include "asm.h"
+#include "capture.h"
 
 /*==================================================================*/
 /* variables globales                                               */
@@ -102,7 +104,7 @@ void display_success_driver (char *driver)
 int init_all ()
 {
   int result=0;
-  int config=1,timer=1,keyboard=1,mouse=1,sound=1,joystick=1,network=1;
+  int assembly=1,config=1,timer=1,keyboard=1,mouse=1,sound=1,joystick=1,network=1;
 
   /*
    * we use srand to garantee that rand() won't return the same values
@@ -111,11 +113,8 @@ int init_all ()
   srand(time(NULL));
 
   set_startup();
-
   log_init();
-
   lw_lang_init();
-
   set_uformat(U_ASCII);
 
   allegro_init ();
@@ -131,6 +130,11 @@ int init_all ()
   set_color_depth(8);
   set_color_conversion(COLORCONV_REDUCE_TO_256);
 
+  /*
+   * We check for potential struct size errors
+   */
+  assembly=lw_asm_check_struct_align();
+
   log_print_str("Loading config options from \"");
   log_print_str(STARTUP_CFG_PATH);
   log_print_str("\"");
@@ -220,11 +224,17 @@ int init_all ()
   log_print_str("Setting up network");
   display_success(network=lw_sock_init());
 
-  /*  set_color_conversion (COLORCONV_TOTAL); */
-  
-  result= timer&&keyboard&&mouse
-    &&(sound||!STARTUP_CHECK)
-    &&(joystick||!STARTUP_CHECK);
+  /*
+   * We update the "capture" mode
+   */
+  lw_capture_set_mode(STARTUP_CAPTURE);
+
+  result= (assembly || !STARTUP_CHECK) 
+    && timer
+    && keyboard
+    && mouse
+    && (sound || !STARTUP_CHECK)
+    && (joystick || !STARTUP_CHECK);
 
   return (result);
 }
