@@ -54,6 +54,7 @@
 
 #include "netkey.h"
 #include "keyexch.h"
+#include "log.h"
 
 /*==================================================================*/
 /* defines                                                          */
@@ -72,7 +73,8 @@ int LW_KEYEXCH_SOCK=-1;
 /* static functions                                                 */
 /*==================================================================*/
 
-static int send_keys(int sock, CURSOR *cursors);
+static int send_keys(int sock, CURSOR *cursors,
+		     unsigned int checksum);
 static int recv_keys(int sock, CURSOR *cursors);
 
 /*==================================================================*/
@@ -80,7 +82,8 @@ static int recv_keys(int sock, CURSOR *cursors);
 /*==================================================================*/
 
 /*------------------------------------------------------------------*/
-static int send_keys(int sock, CURSOR *cursors)
+static int send_keys(int sock, CURSOR *cursors,
+		     unsigned int checksum)
 {
   int result=0;
   int i;
@@ -118,6 +121,12 @@ static int send_keys(int sock, CURSOR *cursors)
       netkey.cmd=LW_NETKEY_CMD_DECREASE_LAG;
     }
 
+  /*
+   * Checksum which will help the server to control things and
+   * detect consistency problems.
+   */
+  netkey.checksum=checksum;
+
   result=lw_netkey_send(sock,&netkey);
 
   return result;
@@ -144,15 +153,29 @@ static int recv_keys(int sock, CURSOR *cursors)
       cursors[i].key_state=netkey.key_states[i];
     }
 
+  if (result) 
+    {
+      /*
+       * Now we check for checksum errors
+       */
+      if (! netkey.checksum)
+	{
+	  log_println_str("Error: incorrect checksum, the game is broken, or someone is cheating");
+	}
+    }      
+
   return result;
 }
 
 /*------------------------------------------------------------------*/
-int lw_keyexch_flush(CURSOR *cursors)
+int lw_keyexch_flush(CURSOR *cursors, unsigned int checksum)
 {
   int result=1;
 
-  result=result && send_keys(LW_KEYEXCH_SOCK,cursors);
+  /*
+   * It's important to send keys before we receive them
+   */
+  result=result && send_keys(LW_KEYEXCH_SOCK,cursors, checksum);
   result=result && recv_keys(LW_KEYEXCH_SOCK,cursors);
 
   return result;
