@@ -71,6 +71,6 @@ extern int LW_KEYEXCH_SOCK;
 /* fonctions globales                                               */
 /*==================================================================*/
 
-int lw_keyexch_flush(CURSOR *cursors);
+int lw_keyexch_flush(CURSOR *cursors, unsigned int checksum);
 
 #endif
