@@ -386,8 +386,8 @@ static void box_for_choose_map (void)
       menu_real_coord (d+i);
       d[i].x--;
       d[i].y--;
-      d[i].w++;
-      d[i].h++;
+      d[i].w+=2;
+      d[i].h+=2;
     }
 
   d[6].proc=NULL; /* 6 remplace par 9 pour boites des sliders */
