@@ -66,6 +66,7 @@
 #include "time.h"
 #include "keyexch.h"
 #include "network.h"
+#include "checksum.h"
 
 /*==================================================================*/
 /* variables globales                                               */
@@ -404,6 +405,7 @@ static void update_key_cpu_cursor (int indice)
 void move_all_cursors (void)
 {
   int i,j;
+  int checksum;
 
   /*
    * polls the module which wraps mouse moves to virtual keys
@@ -455,14 +457,16 @@ void move_all_cursors (void)
        */
       if (LW_NETWORK_ON)
 	{
+	  checksum=lw_checksum_calc(CURRENT_ARMY,CURRENT_ARMY_SIZE);
+
 	  /*
 	   * This call will automatically overwrite the key states
 	   * with those received from the network
 	   */
-	    if (!lw_keyexch_flush(CURRENT_CURSOR))
-	      {
-		LW_NETWORK_ERROR_DETECTED=1;
-	      }
+	  if (!lw_keyexch_flush(CURRENT_CURSOR, checksum))
+	    {
+	      LW_NETWORK_ERROR_DETECTED=1;
+	    }
 	}
 
       for (i=0;i<NB_TEAMS;++i)
