@@ -65,7 +65,7 @@
 /* constants                                                        */
 /*==================================================================*/
 
-#define LW_NETKEY_SERIALIZED_SIZE (NB_TEAMS+2)
+#define LW_NETKEY_SERIALIZED_SIZE (NB_TEAMS+2+4)
 
 /*==================================================================*/
 /* types                                                            */
@@ -75,7 +75,7 @@
  * Used to store the information of a LW_NETKEY struct once it has
  * been serialized to be sent over the network
  */
-typedef char LW_NETKEY_SERIALIZED[LW_NETKEY_SERIALIZED_SIZE];
+typedef unsigned char LW_NETKEY_SERIALIZED[LW_NETKEY_SERIALIZED_SIZE];
 
 /*==================================================================*/
 /* static functions declaration                                     */
@@ -176,6 +176,14 @@ static void encode(LW_NETKEY_SERIALIZED dst, LW_NETKEY *src)
    * We transform the argument since we assume it's a numeric arg
    */
   dst[NB_TEAMS+1]=int_to_ascii_hexa(src->arg);
+
+  /*
+   * Now we serialize the "map checksum"
+   */
+  dst[NB_TEAMS+2+0]=(src->checksum) & 0xFF;
+  dst[NB_TEAMS+2+1]=(src->checksum>>8) & 0xFF;
+  dst[NB_TEAMS+2+2]=(src->checksum>>16) & 0xFF;
+  dst[NB_TEAMS+2+3]=(src->checksum>>24) & 0xFF;
 }
 
 /*------------------------------------------------------------------*/
@@ -205,6 +213,14 @@ static void decode(LW_NETKEY *dst, LW_NETKEY_SERIALIZED src)
    * We transform the argument since we assume it's a numeric arg
    */
   dst->arg=ascii_hexa_to_int(src[NB_TEAMS+1]);
+
+  /*
+   * Now we serialize the "map checksum"
+   */
+  dst->checksum=((unsigned int) src[NB_TEAMS+2+3])<<24
+    | ((unsigned int) src[NB_TEAMS+2+2])<<16
+    | ((unsigned int) src[NB_TEAMS+2+1])<<8
+    | ((unsigned int) src[NB_TEAMS+2+0]);
 }
 
 /*------------------------------------------------------------------*/
