@@ -81,23 +81,24 @@ typedef struct
 #define LW_NETMESS_CODE_ECHO           4
 #define LW_NETMESS_CODE_PROGRAM        5
 #define LW_NETMESS_CODE_VERSION        6
-#define LW_NETMESS_CODE_FREE           7
-#define LW_NETMESS_CODE_TEAM           8
-#define LW_NETMESS_CODE_READY          9
-#define LW_NETMESS_CODE_START         10
-#define LW_NETMESS_CODE_WAITING       11
-#define LW_NETMESS_CODE_NEXT          12
-#define LW_NETMESS_CODE_TEAMSTARTINFO 13
-#define LW_NETMESS_CODE_WHO           14
-#define LW_NETMESS_CODE_LOCAL         15
-#define LW_NETMESS_CODE_NETWORK       16
-#define LW_NETMESS_CODE_NOBODY        17
-#define LW_NETMESS_CODE_SENDMAP       18
-#define LW_NETMESS_CODE_RECVMAP       19
-#define LW_NETMESS_CODE_SENDCONFIG    20
-#define LW_NETMESS_CODE_RECVCONFIG    21
-#define LW_NETMESS_CODE_CHATTALK      22
-#define LW_NETMESS_CODE_CHATLISTEN    23
+#define LW_NETMESS_CODE_PASSWORD       7
+#define LW_NETMESS_CODE_FREE           8
+#define LW_NETMESS_CODE_TEAM           9
+#define LW_NETMESS_CODE_READY         10
+#define LW_NETMESS_CODE_START         11
+#define LW_NETMESS_CODE_WAITING       12
+#define LW_NETMESS_CODE_NEXT          13
+#define LW_NETMESS_CODE_TEAMSTARTINFO 14
+#define LW_NETMESS_CODE_WHO           15
+#define LW_NETMESS_CODE_LOCAL         16
+#define LW_NETMESS_CODE_NETWORK       17
+#define LW_NETMESS_CODE_NOBODY        18
+#define LW_NETMESS_CODE_SENDMAP       19
+#define LW_NETMESS_CODE_RECVMAP       20
+#define LW_NETMESS_CODE_SENDCONFIG    21
+#define LW_NETMESS_CODE_RECVCONFIG    22
+#define LW_NETMESS_CODE_CHATTALK      23
+#define LW_NETMESS_CODE_CHATLISTEN    24
 
 #define LW_NETMESS_TEXT_OK            "OK"
 #define LW_NETMESS_TEXT_ERR           "ERR"
@@ -105,6 +106,7 @@ typedef struct
 #define LW_NETMESS_TEXT_ECHO          "ECHO"
 #define LW_NETMESS_TEXT_PROGRAM       "PROGRAM"
 #define LW_NETMESS_TEXT_VERSION       "VERSION"
+#define LW_NETMESS_TEXT_PASSWORD      "PASSWORD"
 #define LW_NETMESS_TEXT_FREE          "FREE"
 #define LW_NETMESS_TEXT_TEAM          "TEAM"
 #define LW_NETMESS_TEXT_READY         "READY"
