@@ -52,6 +52,7 @@
 /*==================================================================*/
 
 #include <string.h>
+#include <stdlib.h>
 
 #include "back.h"
 #include "base.h"
@@ -73,16 +74,37 @@
 #include "lang.h"
 #include "log.h"
 #include "connect.h"
+#include "thrdgen.h"
+#include "exit.h"
+#include "capture.h"
 
 /*==================================================================*/
-/* variables globales                                               */
+/* constants                                                        */
 /*==================================================================*/
 
+#define LW_NETPLAY_DELAY 10
+
+/*==================================================================*/
+/* structs                                                          */
+/*==================================================================*/
+
+typedef struct {
+  int sock;
+  char address[LW_SOCK_IP_SIZE];
+  int port;
+  int running;
+  int allow_exit;
+  int free_data;
+  int result;
+} LW_NETPLAY_CONNECT_DATA;
+
 /*==================================================================*/
 /* static functions                                                 */
 /*==================================================================*/
 
 static int connect_on_server_start(int *sock, char *ip, int port);
+static int connect_on_server_continue(int sock);
+static void connect_on_server_start_callback(void *arg);
 static int connect_on_server_finish(int sock);
 
 /*==================================================================*/
@@ -106,17 +128,19 @@ int netplay_sequence()
 			      CONFIG_SERVER_ADDRESS,
 			      CONFIG_SERVER_PORT))
     {
-      result=lw_connect_menu(sock);
-
-      if (result==MENU_OK)
+      if (connect_on_server_continue(sock))
 	{
-	  if (connect_on_server_finish(sock))
+	  result=lw_connect_menu(sock);
+	  
+	  if (result==MENU_OK)
 	    {
-	      LW_NETWORK_ON=1;
-	      lw_network_attribute_colors();
-	      lw_network_attribute_parts();
-	      result=play_sequence_ex ();
-	      LW_NETWORK_ON=0;
+	      if (connect_on_server_finish(sock))
+		{
+		  LW_NETWORK_ON=1;
+		  lw_network_attribute_colors();
+		  lw_network_attribute_parts();
+		  result=play_sequence_ex ();
+		}
 	    }
 	}
     }
@@ -129,33 +153,147 @@ int netplay_sequence()
 
 /*------------------------------------------------------------------*/
 /*
- * Opens the socket on the server, sends some messages to the server,
- * and displays message boxes to the player so that he has an idea
- * of what's going on.
+ * Opens the socket on the server. This is a little complex, for we
+ * fire an extra thread so that if things go slow, the player can
+ * use ESCAPE to stop everything.
  */
 static int connect_on_server_start(int *sock, char *address, int port)
 {
   int result=1;
-  int free_teams;
-  //  int waited_teams;
-  int i;
+  LW_NETPLAY_CONNECT_DATA *connect_data;
+  LW_NETPLAY_CONNECT_DATA connect_data_stack;
+  int esc_pressed=0;
+  int dummy;
 
-  (*sock)=-1;
-
-  if (result==1)
+  connect_data=(LW_NETPLAY_CONNECT_DATA *) 
+    malloc(sizeof(LW_NETPLAY_CONNECT_DATA));
+  if (!connect_data)
     {
-      display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_CONNECT));
-      if (!lw_sock_connect(sock,address,port))
+      /*
+       * Couldn't allocate, fallback on stack data
+       */
+      connect_data=&connect_data_stack;
+    }
+
+  connect_data->sock=-1;
+  strncpy(connect_data->address,address,sizeof(connect_data->address));
+  connect_data->address[sizeof(connect_data->address)-1]='\0';
+  connect_data->port=port;
+  connect_data->running=0;
+  connect_data->allow_exit=0;
+  connect_data->free_data=1;
+  connect_data->result=0;
+
+  display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_CONNECT));
+
+  connect_data->running=1;
+  if (connect_data != &connect_data_stack &&      
+      lw_thread_start(connect_on_server_start_callback,
+		      connect_data))
+    {
+      /*
+       * Everything's OK, do nothing!
+       */
+    }
+  else
+    {
+      /*
+       * Memory or thread error, we're in trouble,
+       * we fallback on calling the function in the main thread
+       * with the data allocated in the stack.
+       */      
+      connect_data->free_data=0;
+      connect_on_server_start_callback(connect_data);
+    }
+
+  /*
+   * We wait until the thread is finished and/or the player
+   * pressed ESC
+   */
+  while (connect_data->running && !esc_pressed)
+    {
+      if (key[KEY_ESC])
+	{
+	  esc_pressed=1;
+	}
+      /*
+       * We call keypressed() to force Allegro to call automatically
+       * poll_keyboard() if needed
+       */
+      dummy=keypressed();
+      rest(LW_NETPLAY_DELAY);
+      lw_capture_dump_menu();
+      my_exit_poll();
+    }
+
+  result=connect_data->result;
+  *sock=connect_data->sock;
+
+  if (!result)
+    {
+      *sock=-1;
+      if (!esc_pressed)
 	{
-	  result=0;
 	  error_unable_to_connect();
 	}
     }
+
+  /*
+   * We tell the thread that it can exit whenever it wants,
+   * we don't care, it has its own data...
+   */
+  connect_data->allow_exit=1;
+
+  return result;
+}
+
+/*------------------------------------------------------------------*/
+/*
+ * Callback used by the thread launched by connect_on_server_start
+ * to open the TCP/IP connection with the server
+ */
+static void connect_on_server_start_callback(void *arg)
+{
+  LW_NETPLAY_CONNECT_DATA *connect_data;
+
+  connect_data=(LW_NETPLAY_CONNECT_DATA *) arg;
+
+  connect_data->result=lw_sock_connect(&(connect_data->sock),
+				       connect_data->address,
+				       connect_data->port);
+  connect_data->running=0;
+
+  /*
+   * Now we wait until the main threads tells us we can stop.
+   * It's important for we must not free the data too early...
+   */
+  while (!connect_data->allow_exit)
+    {
+      rest(LW_NETPLAY_DELAY);
+    }
+
+  if (connect_data->free_data)
+    {
+      free(connect_data);
+    }
+}
   
+/*------------------------------------------------------------------*/
+/*
+ * Exchanges some messages with the server,
+ * and displays message boxes to the player so that he has an idea
+ * of what's going on.
+ */
+static int connect_on_server_continue(int sock)
+{
+  int result=1;
+  int free_teams;
+  int i;
+
   if (result==1)
     {
       display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_PING));
-      if (!lw_protocol_do_ping(*sock))
+      if (!lw_protocol_do_ping(sock))
 	{
 	  result=0;
 	  error_not_a_liquidwar_server();
@@ -165,7 +303,7 @@ static int connect_on_server_start(int *sock, char *address, int port)
   if (result==1)
     {
       display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_CHECKPROGRAM));
-      if (!lw_protocol_tell_program(*sock,LW_PROGRAM))
+      if (!lw_protocol_tell_program(sock,LW_PROGRAM))
 	{
 	  result=0;
 	  error_not_a_liquidwar_server();
@@ -174,17 +312,27 @@ static int connect_on_server_start(int *sock, char *address, int port)
   if (result==1)
     {
       display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_CHECKVERSION));
-      if (!lw_protocol_tell_version(*sock,LW_VERSION))
+      if (!lw_protocol_tell_version(sock,LW_VERSION))
 	{
 	  result=0;
 	  error_wrong_version();
 	}
     }
 
+  if (result==1)
+    {
+      display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_SENDPASSWORD));
+      if (!lw_protocol_tell_password(sock,CONFIG_PASSWORD))
+	{
+	  result=0;
+	  error_bad_password();
+	}
+    }
+
   if (result==1)
     {
       display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_FREETEAMS));
-      if (lw_protocol_ask_free_teams(*sock,&free_teams))
+      if (lw_protocol_ask_free_teams(sock,&free_teams))
 	{
 	  if (free_teams<=0)
 	    {
@@ -206,7 +354,7 @@ static int connect_on_server_start(int *sock, char *address, int port)
 	{
 	  if (CONFIG_CONTROL_TYPE[i] != CONFIG_CONTROL_TYPE_OFF)
 	    {
-	      if (lw_protocol_reserve_team(*sock,i,
+	      if (lw_protocol_reserve_team(sock,i,
 					   CONFIG_CONTROL_TYPE[i],
 					   CONFIG_PLAYER_NAME[i]))
 		{
@@ -225,7 +373,7 @@ static int connect_on_server_start(int *sock, char *address, int port)
     {
       display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_SENDCONFIG));
       if (!lw_protocol_send_config
-	  (*sock))
+	  (sock))
 	{
 	  result=0;
 	  error_network_problem();
@@ -236,7 +384,7 @@ static int connect_on_server_start(int *sock, char *address, int port)
     {
       display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_SENDMAP));
       if (!lw_protocol_send_map
-	  (*sock,RAW_MAP_ORDERED[CONFIG_LEVEL_MAP]))
+	  (sock,RAW_MAP_ORDERED[CONFIG_LEVEL_MAP]))
 	{
 	  result=0;
 	  error_cant_send_map();
@@ -246,7 +394,7 @@ static int connect_on_server_start(int *sock, char *address, int port)
   if (result==1)
     {
       display_center_message (lw_lang_string(LW_LANG_STRING_NETPLAY_WAITING));
-      if (!lw_protocol_i_am_ready(*sock))
+      if (!lw_protocol_i_am_ready(sock))
 	{
 	  result=0;
 	  error_network_problem();
@@ -259,12 +407,12 @@ static int connect_on_server_start(int *sock, char *address, int port)
    */
   if (result)
     {
-      LW_KEYEXCH_SOCK=*sock;
+      LW_KEYEXCH_SOCK=sock;
     }
   else
     {
       LW_KEYEXCH_SOCK=-1;
-      lw_sock_close(sock);
+      lw_sock_close(&sock);
     }
   
   return result;
