@@ -131,7 +131,7 @@ int lw_protocol_tell_program (int sock, char *program)
   int result=0;
   char message[LW_SOCK_MESSAGE_SIZE];
 
-  sprintf(message,"%s %s",LW_NETMESS_TEXT_PROGRAM,program);
+  sprintf(message,"%s '%s'",LW_NETMESS_TEXT_PROGRAM,program);
   if (lw_sock_send_str(sock,message))
     {
       if (recv_ok(sock))
@@ -152,7 +152,28 @@ int lw_protocol_tell_version (int sock, char *version)
   int result=0;
   char message[LW_SOCK_MESSAGE_SIZE];
 
-  sprintf(message,"%s %s",LW_NETMESS_TEXT_VERSION,version);
+  sprintf(message,"%s '%s'",LW_NETMESS_TEXT_VERSION,version);
+  if (lw_sock_send_str(sock,message))
+    {
+      if (recv_ok(sock))
+	{
+	  result=1;
+	}
+    }
+
+  return result;
+}
+
+/*------------------------------------------------------------------*/
+/*
+ * Sends the game password to the server.
+ */
+int lw_protocol_tell_password (int sock, char *password)
+{
+  int result=0;
+  char message[LW_SOCK_MESSAGE_SIZE];
+
+  sprintf(message,"%s '%s'",LW_NETMESS_TEXT_PASSWORD,password);
   if (lw_sock_send_str(sock,message))
     {
       if (recv_ok(sock))
