@@ -186,4 +186,11 @@ void play_loose (void)
   my_play_sample (SAMPLE_SFX_LOOSE, CONFIG_SFX_VOLUME);
 }
 
+/*------------------------------------------------------------------*/
+void play_connect (void)
+{
+ if (LOADED_SFX)
+  my_play_sample (SAMPLE_SFX_CONNECT, CONFIG_SFX_VOLUME);
+}
+
 
