@@ -103,8 +103,9 @@ static void reset(LW_SRVCHAN *chan)
   chan->sock=-1;
   chan->ip[0]='\0';
   chan->port=0;
-  chan->program_ok=1; // set to 1 temporary for testing
-  chan->version_ok=1; // set to 1 temporary for testing
+  chan->program_ok=0;
+  chan->version_ok=0;
+  chan->password_ok=0;
   chan->ready_ok=0;
   chan->start=0;
   chan->map=NULL;
@@ -114,6 +115,8 @@ static void reset(LW_SRVCHAN *chan)
       lw_srvteam_reset(&(chan->team[i]));
     }
   lw_chat_reset(&(chan->chat));
+  chan->checksum_recv=0;
+  chan->checksum_send=0;
 }
 
 /*------------------------------------------------------------------*/
@@ -246,6 +249,54 @@ int lw_srvchan_handle_version(int sock, LW_NETMESS *mess,
   return result;
 }
 
+/*------------------------------------------------------------------*/
+/*
+ * handles the "PASSWORD" message
+ */
+int lw_srvchan_handle_password(int sock, LW_NETMESS *mess,
+			       int *ok, char *password)
+{
+  int result=0;
+
+  switch (mess->argc)
+    {
+    case 1:
+      /*
+       * OK, there's an argument, we check if it's the good password.
+       */
+      if (strncmp(mess->argv[0],password,PASSWORD_SIZE)==0)
+	{
+	  (*ok)=1;
+	  result=1;
+	}
+      break;
+    }
+
+  if (!strcmp(password,""))
+    {
+      /*
+       * OK, now the server's password is "", so we consider any
+       * that any password from the client is OK.
+       */
+      (*ok)=1;
+      result=1;
+    }
+
+  /*
+   * Now we send feedback on the network
+   */
+  if (result)
+    {
+      lw_sock_send_str(sock,LW_NETMESS_TEXT_OK);
+    }
+  else
+    {
+      lw_sock_send_str(sock,LW_NETMESS_TEXT_ERR);
+    }
+
+  return result;
+}
+
 /*------------------------------------------------------------------*/
 /*
  * handles the "FREE" message
@@ -782,7 +833,8 @@ int lw_srvchan_handle_unknown(int sock, LW_NETMESS *mess)
  * Waits for a client to connect, returns 1 if successfull, 0 if not
  */
 int lw_srvchan_wait_teams(LW_SRVCHAN *chan, int *free_teams,
-			  int sock,void **map, LW_NETCONF *config)
+			  int sock,void **map, LW_NETCONF *config,
+			  char *password)
 {
   int result=0;
   int ret;
@@ -790,7 +842,6 @@ int lw_srvchan_wait_teams(LW_SRVCHAN *chan, int *free_teams,
   char message[LW_SOCK_MESSAGE_SIZE];
   LW_NETMESS *mess;
   int free_received=0;
-
   reset(chan);
 
   if (lw_sock_accept(&(chan->sock),chan->ip,&(chan->port),sock,1))
@@ -823,13 +874,22 @@ int lw_srvchan_wait_teams(LW_SRVCHAN *chan, int *free_teams,
 		      ret_mess=lw_srvchan_handle_echo(chan->sock,mess);
 		      break;
 		    case LW_NETMESS_CODE_PROGRAM:
-		      ret_mess=lw_srvchan_handle_program(chan->sock,mess,&(chan->program_ok));
+		      ret_mess=lw_srvchan_handle_program(chan->sock,mess,
+							 &(chan->program_ok));
 		      break;
 		    case LW_NETMESS_CODE_VERSION:
-		      ret_mess=lw_srvchan_handle_version(chan->sock,mess,&(chan->version_ok));
+		      ret_mess=lw_srvchan_handle_version(chan->sock,mess,
+							 &(chan->version_ok));
+		      break;
+		    case LW_NETMESS_CODE_PASSWORD:
+		      ret_mess=lw_srvchan_handle_password(chan->sock,mess,
+							  &(chan->password_ok),
+							  password);
 		      break;
 		    case LW_NETMESS_CODE_FREE:
-		      ret_mess=lw_srvchan_handle_free(chan->sock,mess,*free_teams,&free_received);
+		      ret_mess=lw_srvchan_handle_free(chan->sock,mess,
+						      *free_teams,
+						      &free_received);
 		      break;
 		    case LW_NETMESS_CODE_TEAM:
 		      ret_mess=handle_team(chan,mess,free_teams);
@@ -871,6 +931,7 @@ int lw_srvchan_wait_teams(LW_SRVCHAN *chan, int *free_teams,
 
       if (chan->program_ok && 
 	  chan->version_ok && 
+	  chan->password_ok &&
 	  chan->ready_ok &&
 	  ret_mess)
 	{
@@ -1262,6 +1323,7 @@ int lw_srvchan_recv_keys(LW_SRVCHAN *chan, LW_NETKEY *netkey)
 
   netkey->cmd=chan_netkey.cmd;
   netkey->arg=chan_netkey.arg;
+  chan->checksum_recv=chan_netkey.checksum;
 
   return result;
 }
@@ -1291,6 +1353,7 @@ int lw_srvchan_send_keys(LW_SRVCHAN *chan, LW_NETKEY *netkey)
 
   chan_netkey.cmd=netkey->cmd;
   chan_netkey.arg=netkey->arg;
+  chan_netkey.checksum=chan->checksum_send;
 
   result=lw_netkey_send(chan->sock,&chan_netkey);
 
