@@ -54,6 +54,7 @@
 
 #include <stdlib.h>
 #include <sys/types.h>
+#include <string.h>
 
 #include "srvcont.h"
 #include "srvchan.h"
@@ -75,6 +76,7 @@ typedef struct
   char ip[LW_SOCK_IP_SIZE];
   int port;
   int *thread_running;
+  char password[PASSWORD_SIZE+1];
 } LW_SRVCONT_REJECT_DATA;
 
 /*==================================================================*/
@@ -99,7 +101,7 @@ static void find_chan_and_team_by_server_id(LW_SRVCONT *cont,
 					    int *chan,
 					    int *team);
 static int send_lag_key_presses(LW_SRVCONT *cont,int lag);
-static int reject_new_clients(LW_SRVCONT *cont);
+static int reject_new_clients(LW_SRVCONT *cont, char *password);
 static void reject_new_clients_callback(void *args);
 
 /*------------------------------------------------------------------*/
@@ -314,7 +316,8 @@ static void prepare_team_start_info(LW_TEAMSTARTINFO *team_start_info,
  */
 int lw_srvcont_wait_teams(LW_SRVCONT *cont, 
 			  LW_WWWSRV_DATA *www_data,
-			  int nb_teams, int port)
+			  int nb_teams, int port,
+			  char *password)
 {
   int free_teams=0;
   int result=0;
@@ -378,7 +381,8 @@ int lw_srvcont_wait_teams(LW_SRVCONT *cont,
 				    &free_teams,
 				    cont->sock,
 				    &(cont->chan[cont->nb_chans].map),
-				    &(cont->chan[cont->nb_chans].config)))
+				    &(cont->chan[cont->nb_chans].config),
+				    password))
 	    {
 	      cont->nb_teams=nb_teams-free_teams;
 	      ++(cont->nb_chans);
@@ -610,10 +614,16 @@ static int send_lag_key_presses(LW_SRVCONT *cont,int lag)
   LW_NETKEY netkey;
 
   lw_netkey_reset(&netkey);
+
   for (j=0;j<lag && result;++j)
     {
       for (i=0;i<cont->nb_chans && result;++i)
 	{
+	  /*
+	   * We put the checksum to 1, to inform the clients that
+	   * everything is OK
+	   */
+	  cont->chan[i].checksum_send=1;
 	  if (!lw_srvchan_send_keys(&(cont->chan[i]),&netkey))
 	    {
 	      result=0;
@@ -624,13 +634,60 @@ static int send_lag_key_presses(LW_SRVCONT *cont,int lag)
   return result;
 }
 
+/*------------------------------------------------------------------*/
+/* 
+ * Checks if the checksums from the clients are all the same
+ */
+int check_checksum(LW_SRVCONT *cont)
+{
+  int i,j;
+  int ok=1;
+  int nb_diff;
+
+  for (i=0;i<cont->nb_chans;++i)
+    {
+      cont->chan[i].checksum_send=1;
+      nb_diff=0;
+      for (j=0;j<cont->nb_chans;++j)
+	{
+	  if (cont->chan[i].checksum_recv != cont->chan[j].checksum_recv)
+	    {
+	      ok=0;
+	      nb_diff++;
+
+	      /*
+	       * Now if this channels has a different checksum
+	       * than everyone else, we consider there's an error
+	       * on it. But if there's at least one channel that
+	       * has the same checksum, we consider it's OK
+	       * (for this channel only of course...)
+	       */
+	      if (nb_diff>=cont->nb_chans-1)
+		{
+		  cont->chan[i].checksum_send=0;
+
+		  log_print_str("Checksum error on client \"");
+		  log_print_str(cont->chan[i].ip);
+		  log_print_str(":");
+		  log_print_int(cont->chan[i].port);
+		  log_print_str("\"");
+		  log_println();
+		}
+	    }
+	}
+    }
+
+  return ok;
+}
+
 /*------------------------------------------------------------------*/
 /* 
  * Sends a final OK message to all the clients
  */
 int lw_srvcont_replicate_keys(LW_SRVCONT *cont,
 			      LW_WWWSRV_DATA *www_data,
-			      int lag)
+			      int lag,
+			      char *password)
 {
   int result=1;
   LW_NETKEY netkey;
@@ -646,70 +703,170 @@ int lw_srvcont_replicate_keys(LW_SRVCONT *cont,
   int average_lag=0;
   time_t start,start2,end;
   double elapsed;
+  int lag_key_presses_to_send=0;
+  int skip_keys;
 
   www_data->log=1;
   lw_wwwsrv_register(www_data);
 
-  result=send_lag_key_presses(cont,lag);
+  lag_key_presses_to_send=lag;
 
-  if (!result)
-    {
-      log_print_str("Error sending lag keys, disconnecting everyone");
-      log_println();
-    }
-  else
-    {
-      start2=start=lw_srvtime_seconds();
+  start2=start=lw_srvtime_seconds();
 
-      while (cont->nb_chans>0)
+  while (cont->nb_chans>0)
+    {
+      lw_netkey_reset(&netkey);
+
+      /*
+       * We still listen on the listening socket, to reject
+       * newcomers since the game already started
+       */
+      reject_new_clients(cont,password);
+
+      all_clients_request_increase_lag=1;
+      all_clients_request_decrease_lag=1;
+      a_client_requests_increase_lag=0;
+      a_client_requests_decrease_lag=0;
+
+      /*
+       * First we get the informations from all the players
+       */
+      for (i=cont->nb_chans-1;i>=0;--i)
 	{
-	  lw_netkey_reset(&netkey);
+	  if (lw_srvchan_recv_keys(&(cont->chan[i]),&netkey))
+	    {
+	      /*
+	       * if at least a client does not request a lag
+	       * increase, then we don't do it
+	       */
+	      if (netkey.cmd==LW_NETKEY_CMD_INCREASE_LAG)
+		{
+		  a_client_requests_increase_lag=1;
+		}
+	      else
+		{
+		  all_clients_request_increase_lag=0;
+		}
 
+	      /*
+	       * if at least a client does not request a lag
+	       * decrease, then we don't do it
+	       */
+	      if (netkey.cmd==LW_NETKEY_CMD_DECREASE_LAG)
+		{
+		  a_client_requests_decrease_lag=1;
+		}
+	      else
+		{
+		  all_clients_request_decrease_lag=0;
+		}
+	    }
+	  else
+	    {
+	      /*
+	       * There's been an error receiving data from this team
+	       * so we decide to "close" it.
+	       */
+	      log_print_str("Disconnecting \"");
+	      log_print_str(cont->chan[i].ip);
+	      log_print_str(":");
+	      log_print_int(cont->chan[i].port);
+	      log_print_str("\"");
+	      log_println();
+
+	      lw_srvchan_close(&(cont->chan[i]));
+	      for (j=i+1;j<cont->nb_chans;++j)
+		{
+		  cont->chan[j-1]=cont->chan[j];
+		}
+	      cont->nb_chans--;
+	    }
+	}
+
+      /*
+       * we reset the cmd and arg fields
+       */
+      netkey.cmd=LW_NETKEY_CMD_NONE;
+      netkey.arg=0;
+
+      /*
+       * We check if the checksums are OK
+       */
+      check_checksum(cont);
+
+      /*
+       * we use a counter system, so that all client must answer
+       * "yes we want a lag increase" several times in a row
+       * before we actually update the lag
+       */
+      if (all_clients_request_increase_lag)
+	{
+	  increase_lag_counter++;
+	}
+      else
+	{
+	  increase_lag_counter=0;
+	}
+
+      /*
+       * we use a counter system, so that all client must answer
+       * "yes we want a lag decrease" several times in a row
+       * before we actually update the lag
+       */
+      if (a_client_requests_decrease_lag) 
+	{
+	  decrease_lag_counter++;
+	}
+      else
+	{
+	  decrease_lag_counter=0;
+	}
+
+      /*
+       * If we need to increase the lag, we send blank key presses
+       */
+      if (increase_lag_counter>=LW_SRVCONT_INCREASE_LAG_LIMIT
+	  && lag_key_presses_to_send==0)
+	{
+	  increase_lag_counter=0;
+	  lag++;
+	  lag_key_presses_to_send++;
+	}
+
+
+      /*
+       * If we need to decrease the lag, we simply do not send
+       * the key presses...
+       */
+      skip_keys=0;
+      if (decrease_lag_counter>=LW_SRVCONT_DECREASE_LAG_LIMIT)
+	{
+	  decrease_lag_counter=0;
+	  lag--;
+	  if (lag_key_presses_to_send>0)
+	    {
+	      /*
+	       * We cancel the planified lag keys
+	       */
+	      lag_key_presses_to_send--;
+	    }
+	  else
+	    {
+	      /*
+	       * No lag keys planned, we skip the current keys
+	       */
+	      skip_keys=1;
+	    }
+	}
+      
+      if (!skip_keys)
+	{
 	  /*
-	   * We still listen on the listening socket, to reject
-	   * newcomers since the game already started
-	   */
-	  reject_new_clients(cont);
-
-	  all_clients_request_increase_lag=1;
-	  all_clients_request_decrease_lag=1;
-	  a_client_requests_increase_lag=0;
-	  a_client_requests_decrease_lag=0;
-
-	  /*
-	   * First we get the informations from all the players
+	   * Now we send the information to all the players
 	   */
 	  for (i=cont->nb_chans-1;i>=0;--i)
 	    {
-	      if (lw_srvchan_recv_keys(&(cont->chan[i]),&netkey))
-		{
-		  /*
-		   * if at least a client does not request a lag
-		   * increase, then we don't do it
-		   */
-		  if (netkey.cmd==LW_NETKEY_CMD_INCREASE_LAG)
-		    {
-		      a_client_requests_increase_lag=1;
-		    }
-		  else
-		    {
-		      all_clients_request_increase_lag=0;
-		    }
-
-		  /*
-		   * if at least a client does not request a lag
-		   * decrease, then we don't do it
-		   */
-		  if (netkey.cmd==LW_NETKEY_CMD_DECREASE_LAG)
-		    {
-		      a_client_requests_decrease_lag=1;
-		    }
-		  else
-		    {
-		      all_clients_request_decrease_lag=0;
-		    }
-		}
-	      else
+	      if (!lw_srvchan_send_keys(&(cont->chan[i]),&netkey))
 		{
 		  /*
 		   * There's been an error receiving data from this team
@@ -730,127 +887,54 @@ int lw_srvcont_replicate_keys(LW_SRVCONT *cont,
 		  cont->nb_chans--;
 		}
 	    }
+	}
 
-	  /*
-	   * we reset the cmd and arg fields
-	   */
-	  netkey.cmd=LW_NETKEY_CMD_NONE;
-	  netkey.arg=0;
+      /*
+       * We send the lag key presses 1 by 1. This is a way
+       * to synchronize correctly at the beginning of a game.
+       */
+      if (lag_key_presses_to_send>0)
+	{
+	  lag_key_presses_to_send--;
+	  send_lag_key_presses(cont,1);
+	}
 
-	  /*
-	   * we use a counter system, so that all client must answer
-	   * "yes we want a lag increase" several times in a row
-	   * before we actually update the lag
-	   */
-	  if (all_clients_request_increase_lag)
+      rounds++;
+      last_rounds++;
+      average_lag+=lag;
+
+      end=lw_srvtime_seconds();
+      elapsed = end-start;
+
+      if (((int) elapsed)>LW_SRVCONT_DISPLAY_STATE_DELAY)
+	{
+	  if (rounds<=0)
 	    {
-	      increase_lag_counter++;
+	      rounds=1;
 	    }
-	  else
+	  if (last_rounds<=0)
 	    {
-	      increase_lag_counter=0;
+	      last_rounds=1;
 	    }
 
-	  /*
-	   * we use a counter system, so that all client must answer
-	   * "yes we want a lag decrease" several times in a row
-	   * before we actually update the lag
-	   */
-	  if (a_client_requests_decrease_lag) 
-	    {
-	      decrease_lag_counter++;
-	    }
-	  else
-	    {
-	      decrease_lag_counter=0;
-	    }
+	  log_print_int(rounds);
+	  log_print_str(" rounds, ");
+	  log_print_int(last_rounds/((int) elapsed));
+	  log_print_str(" rounds/sec, average lag is ");
+	  log_print_int(average_lag/last_rounds);
+	  log_println();
 
-	  /*
-	   * If we need to increase the lag, we send blank key presses
-	   */
-	  if (increase_lag_counter>=LW_SRVCONT_INCREASE_LAG_LIMIT)
-	    {
-	      increase_lag_counter=0;
-	      lag++;
-	      send_lag_key_presses(cont,1);
-	    }
+	  start=end;
+	  last_rounds=0;
+	  average_lag=0;
+	}
 
-	  /*
-	   * If we need to decrease the lag, we simply do not send
-	   * the key presses...
-	   */
-	  if (decrease_lag_counter>=LW_SRVCONT_DECREASE_LAG_LIMIT)
-	    {
-	      decrease_lag_counter=0;
-	      lag--;
-	    }
-	  else
-	    {
-	      /*
-	       * Now we send the information to all the players
-	       */
-	      for (i=cont->nb_chans-1;i>=0;--i)
-		{
-		  if (!lw_srvchan_send_keys(&(cont->chan[i]),&netkey))
-		    {
-		      /*
-		       * There's been an error receiving data from this team
-		       * so we decide to "close" it.
-		       */
-		      log_print_str("Disconnecting \"");
-		      log_print_str(cont->chan[i].ip);
-		      log_print_str(":");
-		      log_print_int(cont->chan[i].port);
-		      log_print_str("\"");
-		      log_println();
+      if (end-start2>LW_SRVCONT_REGISTER_DELAY)
+	{
+	  start2+=LW_SRVCONT_REGISTER_DELAY;
 
-		      lw_srvchan_close(&(cont->chan[i]));
-		      for (j=i+1;j<cont->nb_chans;++j)
-			{
-			  cont->chan[j-1]=cont->chan[j];
-			}
-		      cont->nb_chans--;
-		    }
-		}
-	    }
-
-	  rounds++;
-	  last_rounds++;
-	  average_lag+=lag;
-
-	  end=lw_srvtime_seconds();
-	  elapsed = end-start;
-
-	  if (((int) elapsed)>LW_SRVCONT_DISPLAY_STATE_DELAY)
-	    {
-	      if (rounds<=0)
-		{
-		  rounds=1;
-		}
-	      if (last_rounds<=0)
-		{
-		  last_rounds=1;
-		}
-
-	      log_print_int(rounds);
-	      log_print_str(" rounds, ");
-	      log_print_int(last_rounds/((int) elapsed));
-	      log_print_str(" rounds/sec, average lag is ");
-	      log_print_int(average_lag/last_rounds);
-	      log_println();
-
-	      start=end;
-	      last_rounds=0;
-	      average_lag=0;
-	    }
-
-	  if (end-start2>LW_SRVCONT_REGISTER_DELAY)
-	    {
-	      start2+=LW_SRVCONT_REGISTER_DELAY;
-
-	      www_data->log=0;
-	      lw_wwwsrv_register(www_data);
-	    }
+	  www_data->log=0;
+	  lw_wwwsrv_register(www_data);
 	}
     }
 
@@ -883,7 +967,7 @@ void lw_srvcont_close(LW_SRVCONT *cont)
  * Launches a thread which will handle new connections while
  * other players are playing.
  */
-static int reject_new_clients(LW_SRVCONT *cont)
+static int reject_new_clients(LW_SRVCONT *cont, char *password)
 {
   int result=0;
   static LW_SRVCONT_REJECT_DATA data;
@@ -902,6 +986,8 @@ static int reject_new_clients(LW_SRVCONT *cont)
       data.ip[0]='\0';
       data.port=0;
       data.thread_running=&thread_running;
+      strncpy(data.password,password,PASSWORD_SIZE);
+      data.password[PASSWORD_SIZE]='\0';
 
       if (lw_sock_accept(&(data.sock),data.ip,&(data.port),cont->sock,0))
 	{
@@ -943,6 +1029,7 @@ static void reject_new_clients_callback(void *args)
   LW_NETMESS *mess;
   int program=0;
   int version=0;
+  int password=0;
   int free_received=0;
 
   memcpy(&data, args, sizeof(LW_SRVCONT_REJECT_DATA));
@@ -980,6 +1067,9 @@ static void reject_new_clients_callback(void *args)
 		case LW_NETMESS_CODE_VERSION:
 		  ret_mess=lw_srvchan_handle_version(data.sock,mess,&version);
 		  break;
+		case LW_NETMESS_CODE_PASSWORD:
+		  ret_mess=lw_srvchan_handle_password(data.sock,mess,&password,data.password);
+		  break;
 		case LW_NETMESS_CODE_FREE:
 		  ret_mess=lw_srvchan_handle_free(data.sock,mess,0,&free_received);
 		  break;
