@@ -86,6 +86,8 @@
 #define IDENT_TEX                 "tex"
 #define IDENT_MID                 "mid"
 #define IDENT_METASERVER          "metaserver"
+#define IDENT_CAPTURE             "capture"
+#define IDENT_TOMBOLA             "tombola"
 
 #define STARTUP_MAX_PATH_LENGTH           1000
 #define STARTUP_MAX_METASERVER_LENGTH      200
@@ -109,6 +111,8 @@ int STARTUP_SVGA=1;
 int STARTUP_STOP=0;
 int STARTUP_AUTO=0;
 int STARTUP_CHECK=0;
+int STARTUP_CAPTURE=0;
+int STARTUP_TOMBOLA=0;
 
 #ifdef ALLEGRO_UNIX
 static char *DEFAULT_CFG_PATH=CONFIG_UNIX_CFG;
@@ -359,6 +363,19 @@ static void set_metaserver (void)
     strcpy(STARTUP_METASERVER,DEFAULT_METASERVER);   
 }
 
+/*------------------------------------------------------------------*/
+static void set_misc (void)
+{
+  if (exist_argument(IDENT_CAPTURE))
+    {
+      STARTUP_CAPTURE=1;
+    }
+  if (exist_argument(IDENT_TOMBOLA))
+    {
+      STARTUP_TOMBOLA=1;
+    }
+}
+
 /*------------------------------------------------------------------*/
 static void set_various_hacks (void)
 {
@@ -385,6 +402,7 @@ void set_startup (void)
   set_auto_check();
   set_bigdata_size();
   set_metaserver();
+  set_misc();
   set_various_hacks();
 }
 
