@@ -60,14 +60,14 @@
 #include "gfxmode.h"
 #include "palette.h"
 #include "viewport.h"
+#include "info.h"
+#include "distor.h"
+#include "log.h"
 
 /*==================================================================*/
 /* variables globales                                               */
 /*==================================================================*/
 
-#define VIEWPORT_SCALE_W 76800
-#define VIEWPORT_SCALE_H 57600
-
 int PAGE_FLIP_H=0;
 BITMAP *NEXT_SCREEN=NULL;
 
@@ -76,6 +76,8 @@ static int VIEWPORT_Y=0;
 static int VIEWPORT_W=0;
 static int VIEWPORT_H=0;
 
+static int VIEWPORT_CHANGE_REGISTERED=0;
+
 /*==================================================================*/
 /* fonctions                                                        */
 /*==================================================================*/
@@ -83,129 +85,152 @@ static int VIEWPORT_H=0;
 /*------------------------------------------------------------------*/
 void rect_for_viewport (void)
 {
- rect (screen,VIEWPORT_X-1,VIEWPORT_Y-1+PAGE_FLIP_H,
-      VIEWPORT_X+VIEWPORT_W,
-      VIEWPORT_Y+VIEWPORT_H+PAGE_FLIP_H,
-      MENU_FG);
+  rect (screen,VIEWPORT_X-1,VIEWPORT_Y-1+PAGE_FLIP_H,
+	VIEWPORT_X+VIEWPORT_W,
+	VIEWPORT_Y+VIEWPORT_H+PAGE_FLIP_H,
+	MENU_FG);
 }
 
 /*------------------------------------------------------------------*/
-static void init_viewport_size (void)
+static void init_viewport_size (int x, int y, int w,int h)
 {
- if (CONFIG_VIEWPORT_SIZE>=8)
+  int scale_w;
+  int scale_h;
+
+  static int viewport_sizes[7]={18,24,27,30,33,35,36};
+
+  scale_w=10*w;
+  scale_h=10*h;
+
+  if (CONFIG_VIEWPORT_SIZE>=8)
     {
-     VIEWPORT_W=VIEWPORT_SCALE_W;
-     VIEWPORT_H=VIEWPORT_SCALE_H;
+      VIEWPORT_W=scale_w;
+      VIEWPORT_H=scale_h;
     }
 
- if (CONFIG_VIEWPORT_SIZE>0 && CONFIG_VIEWPORT_SIZE<8)
+  if (CONFIG_VIEWPORT_SIZE>0 && CONFIG_VIEWPORT_SIZE<8)
     {
-     if (VIEWPORT_SCALE_W*CURRENT_AREA_H
-        <VIEWPORT_SCALE_H*CURRENT_AREA_W)
+      if (scale_w*CURRENT_AREA_H
+	  <scale_h*CURRENT_AREA_W)
         {
-         VIEWPORT_W=(VIEWPORT_SCALE_W*(5+CONFIG_VIEWPORT_SIZE))/12;
-         VIEWPORT_H=(VIEWPORT_W*CURRENT_AREA_H)/CURRENT_AREA_W;
+	  VIEWPORT_W=(scale_w*(viewport_sizes[CONFIG_VIEWPORT_SIZE-1]))
+	    /viewport_sizes[6];
+	  VIEWPORT_H=(VIEWPORT_W*CURRENT_AREA_H)/CURRENT_AREA_W;
         }
-     else
+      else
         {
-         VIEWPORT_H=(VIEWPORT_SCALE_H*(5+CONFIG_VIEWPORT_SIZE))/12;
-         VIEWPORT_W=(VIEWPORT_H*CURRENT_AREA_W)/CURRENT_AREA_H;
+	  VIEWPORT_H=(scale_h*(viewport_sizes[CONFIG_VIEWPORT_SIZE-1]))
+	    /viewport_sizes[6];
+	  VIEWPORT_W=(VIEWPORT_H*CURRENT_AREA_W)/CURRENT_AREA_H;
         }
     }
 
- if (CONFIG_VIEWPORT_SIZE<=0)
+  if (CONFIG_VIEWPORT_SIZE<=0)
     {
-     VIEWPORT_W=CURRENT_AREA_W;
-     VIEWPORT_H=CURRENT_AREA_H;
-     while (VIEWPORT_W>SCREEN_W
-           || VIEWPORT_H>SCREEN_H)
-           {
-            VIEWPORT_W/=2;
-            VIEWPORT_H/=2;
-           }
+      VIEWPORT_W=CURRENT_AREA_W;
+      VIEWPORT_H=CURRENT_AREA_H;
+      while (VIEWPORT_W>w
+	     || VIEWPORT_H>h)
+	{
+	  VIEWPORT_W/=2;
+	  VIEWPORT_H/=2;
+	}
     }
- else
+  else
     {
-     VIEWPORT_W*=SCREEN_W;
-     VIEWPORT_H*=SCREEN_H;
+      VIEWPORT_W*=w;
+      VIEWPORT_H*=h;
 
-     VIEWPORT_W/=VIEWPORT_SCALE_W;
-     VIEWPORT_H/=VIEWPORT_SCALE_H;
+      VIEWPORT_W/=scale_w;
+      VIEWPORT_H/=scale_h;
     }
 
- VIEWPORT_X=((SCREEN_W-VIEWPORT_W)/8)*4;
- VIEWPORT_Y=(SCREEN_H-VIEWPORT_H)/2;
+  VIEWPORT_X=x+((w-VIEWPORT_W)/8)*4;
+  VIEWPORT_Y=y+(h-VIEWPORT_H)/2;
 }
 
 /*------------------------------------------------------------------*/
 BITMAP *page_flip (void)
 {
- int old_h,new_h;
- int flip_allowed=1;
+  int old_h,new_h;
+  int x,y,w,h;
 
- #ifdef ALLEGRO_WINDOWS
-   flip_allowed=0;
- #endif
-   
- if (CONFIG_PAGE_FLIP && flip_allowed && VIRTUAL_H>=(2*SCREEN_H))
-   {
-    if (NEXT_SCREEN)
-       destroy_bitmap (NEXT_SCREEN);
-    else
-        init_viewport_size ();
+  if (CONFIG_PAGE_FLIP && VIRTUAL_H>=(2*SCREEN_H))
+    {
+      if (NEXT_SCREEN)
+	{
+	  destroy_bitmap (NEXT_SCREEN);
+	}
 
-    old_h=PAGE_FLIP_H;
-    new_h=old_h ? 0 : SCREEN_H;
+      lw_info_get_room_for_viewport (&x,&y,&w,&h);
+      init_viewport_size (x,y,w,h);
 
-    scroll_screen (0,old_h);
-    set_clip (screen,0,new_h,SCREEN_W-1,new_h+SCREEN_H-1);
+      old_h=PAGE_FLIP_H;
+      new_h=old_h ? 0 : SCREEN_H;
 
-    PAGE_FLIP_H=new_h;
+      scroll_screen (0,old_h);
+      set_clip (screen,0,new_h,SCREEN_W-1,new_h+SCREEN_H-1);
+
+      PAGE_FLIP_H=new_h;
+
+      NEXT_SCREEN=create_sub_bitmap (screen, VIEWPORT_X,
+				     VIEWPORT_Y+new_h,
+				     VIEWPORT_W,
+				     VIEWPORT_H);
+    }
+  else
+    {
+      PAGE_FLIP_H=0;
+      if (VIEWPORT_CHANGE_REGISTERED || !NEXT_SCREEN)
+	{
+	  scroll_screen (0,0);
+	  set_clip (screen,0,0,SCREEN_W-1,SCREEN_H-1);
+
+	  lw_info_get_room_for_viewport (&x,&y,&w,&h);
+          init_viewport_size(x,y,w,h);
 
-    NEXT_SCREEN=create_sub_bitmap (screen, VIEWPORT_X,
-                                           VIEWPORT_Y+new_h,
-                                           VIEWPORT_W,
-                                           VIEWPORT_H);
-   }
- else
-     {
-	  PAGE_FLIP_H=0;
-      if (!NEXT_SCREEN)
-         {
-		  if (flip_allowed){
-			scroll_screen (0,0);
-			set_clip (screen,0,0,SCREEN_W-1,SCREEN_H-1);
-		  }
-          init_viewport_size();
           NEXT_SCREEN=create_sub_bitmap (screen, VIEWPORT_X,
-                                                 VIEWPORT_Y,
-                                                 VIEWPORT_W,
-                                                 VIEWPORT_H);
-	  }
-     }
- return (NEXT_SCREEN);
+					 VIEWPORT_Y,
+					 VIEWPORT_W,
+					 VIEWPORT_H);
+	}
+    }
+
+  if (VIEWPORT_CHANGE_REGISTERED>0)
+    {
+      rect_for_viewport();
+      init_distorsion_displayer();
+      VIEWPORT_CHANGE_REGISTERED--;
+    }
+
+  return (NEXT_SCREEN);
 }
 
 /*------------------------------------------------------------------*/
 void last_flip (void)
 {
- if (CONFIG_PAGE_FLIP)
- {
-  set_clip (screen,0,0,SCREEN_W-1,SCREEN_H-1);
-  if (!PAGE_FLIP_H)
-     {
-      blit (screen,screen,0,SCREEN_H,0,0,SCREEN_W,SCREEN_H);
-      scroll_screen (0,0);
-     }
-  else
-      PAGE_FLIP_H=0;
- }
+  if (CONFIG_PAGE_FLIP && VIRTUAL_H>=(2*SCREEN_H))
+    {
+      set_clip (screen,0,0,SCREEN_W-1,SCREEN_H-1);
+      if (!PAGE_FLIP_H)
+	{
+	  blit (screen,screen,0,SCREEN_H,0,0,SCREEN_W,SCREEN_H);
+	  scroll_screen (0,0);
+	}
+      else
+	PAGE_FLIP_H=0;
+    }
 
- if (NEXT_SCREEN)
-   {
-    destroy_bitmap (NEXT_SCREEN);
-    NEXT_SCREEN=NULL;
-   }
+  if (NEXT_SCREEN)
+    {
+      destroy_bitmap (NEXT_SCREEN);
+      NEXT_SCREEN=NULL;
+    }
 }
 
+/*------------------------------------------------------------------*/
+void lw_viewport_register_change(int count)
+{
+  VIEWPORT_CHANGE_REGISTERED=count;
+}
 
