@@ -125,7 +125,7 @@ int wave_options (void)
   display_back_image();
   while (retour==0)
     {
-      dp=init_dialog (d,choix);
+      dp=my_init_dialog (d,choix);
       while (my_update_dialog(dp))
 	;
       choix=shutdown_dialog (dp);
