@@ -54,12 +54,33 @@
 /* includes                                                         */
 /*==================================================================*/
 
+#include "sockgen.h"
+#include "base.h"
+
+/*==================================================================*/
+/* constants                                                        */
+/*==================================================================*/
+
 /*==================================================================*/
 /* structs                                                          */
 /*==================================================================*/
 
+typedef struct
+{
+  char label[LW_SOCK_IP_SIZE+1+33];
+  char ip[LW_SOCK_IP_SIZE];
+  int port;
+  int uptime;
+  int busy_players;
+  int max_players;
+  int password;
+  char comment[COMMENT_SIZE+1];
+} LW_WWWCLI_DATA;
+
 /*==================================================================*/
 /* fonctions globales                                               */
 /*==================================================================*/
 
+int lw_wwwcli_get_games(LW_WWWCLI_DATA *buffer, int *size);
+
 #endif
